@@ -1,6 +1,6 @@
 # Copyright (c) 2010 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id: headers.py 42342 2010-05-25 09:49:05Z sylvain $
+# $Id$
 
 
 def format_http_header(name):
@@ -19,6 +19,10 @@ class HTTPHeaders(dict):
     def __getitem__(self, key):
         return dict.__getitem__(self, key.lower())
 
+    def update(self, values):
+        for key, value in values:
+            dict.__setitem__(self, key.lower(), value)
+
     def get(self, key, default=None):
         return dict.get(self, key.lower(), default)
 
