@@ -1,6 +1,6 @@
 # Copyright (c) 2010 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id: log.py 43303 2010-07-05 15:13:59Z sylvain $
+# $Id$
 
 from cgi import escape
 from datetime import datetime
@@ -8,7 +8,6 @@ import sys
 import logging
 import collections
 
-from Acquisition import aq_base
 from zExceptions.ExceptionFormatter import format_exception
 from zope.browser.interfaces import IView
 from zope.interface import Interface
@@ -36,8 +35,14 @@ class ErrorLogView(grok.View):
     grok.require('zope2.ViewManagementScreens')
 
     def update(self):
-        self.errors = reporter.get_last_errors()
+        if 'ignore_errors_update' in self.request.form:
+            reporter.ignore_errors = self.request.form.get(
+                'ignore_errors', [])
 
+        self.all_errors = reporter.all_ignored_errors
+        self.ignored_errors = reporter.ignore_errors
+        self.errors = reporter.get_last_errors()
+        self.debug_mode = self.request.response.debug_mode
 
 
 class ErrorSupplement(object):
@@ -68,11 +73,20 @@ class ErrorSupplement(object):
 class ErrorReporter(object):
     """Utility to help error reporting.
     """
+    all_ignored_errors = [
+            'NotFound', 'Redirect', 'Unauthorized', 'BrokenReferenceError']
 
     def __init__(self):
-        self.__last_errors = collections.deque([], 20)
-        self.__loggable_errors = [
-            'NotFound', 'Redirect', 'Unauthorized', 'BrokenReferenceError']
+        self.__last_errors = collections.deque([], 25)
+        self.__ignore_errors = self.all_ignored_errors
+
+    @apply
+    def ignore_errors():
+        def getter(self):
+            return self.__ignore_errors
+        def setter(self, value):
+            self.__ignore_errors = set(value).intersection(set(self.all_ignored_errors))
+        return property(getter, setter)
 
     def get_last_errors(self):
         """Return all last errors.
@@ -85,14 +99,13 @@ class ErrorReporter(object):
         """Tells you if this error is loggable.
         """
         error_name = error.__class__.__name__
-        return error_name not in self.__loggable_errors
+        return error_name not in self.__ignore_errors
 
     def log_last_error(self, request, response, obj=None, extra=None):
         """Build an error report and log the last available error.
         """
         error_type, error_value, traceback = sys.exc_info()
-        if ((not extra) and
-            (not response.debug_mode) and
+        if ((not response.debug_mode) and
             (not self.is_loggable(error_value))):
             return
 
